# Hyperlink Advanced Targets
It can be a major convenience for webpages to load additional data into their display in response to user interaction, crucially users to continue interacting with it in the meantime. However such a capability can invade a users privacy if it is done without their knowledge or consent.

This standard defines a declarative interface to define these rich interfaces in raw HTML. Thereby allowing user agents to inform users of any requests it will trigger on their behalf.

## Additional `<a>` attributes
The `<a>` element is extended with three new attributes: `prepend`, `replace`, and `append` (collectively "advanced target attributes"). These specify CSS selectors identifying elements on the page into which the response from the link should be loaded. As such when a response is received, it must follow the steps in [Handling responses for advanced targets](https://tildegit.org/alcinnz/PostJS/src/branch/master/drafts/hyperlinks-advanced-targets.md#handling-responses-for-advanced-targets). If multiple elements are selected some of which have an `id` attribute, those should be fetched using seperate requests specifying that `id` in the query parameter `html.target.id`.

`prepend` indicates it should come before any elements already in the target. `append` indicates it should come after any elements already in the target. And `replace` indicates the target element should be emptied immediately before the element is loaded into it. The CSS selector SHOULD be resolved whenever the link's activated.

Additionally `trigger="load"` can be specified on links with a `prepend`, `replace`, or `append` attribute in order to have it download it's resource as seen as it's parsed, and parse it into the HTML tree as soon as a response is received. `trigger="activate"` refers to the default behaviour.

### Additional form input attributes
`<form>` elements, and all the elements associated with them (e.g. `<input>`, `<button>`, `<textarea>`) may use any of the attributes defined for `<a>` elements above. They may all also handle the following attributes:

* `formaction`/`action`, `formmethod`/`method`: Defines the URL and HTTP method that should be sent to the server.
* `inputs`: Defines the names of any inputs (if any) that whose name, checkedness, and values should be sent to the server. Defaults to all of the inputs for the current form.
* `trigger="change"`: Indicates a request should be sent for each batch (as defined by individual user agents) of changes made to the value of the form input.

Additionally the `prepend`/`replace`/`append` attributes are extended so that the `:disabled` and `:invalid` pseudoclasses can be targetted to control respectively whether the input can be interacted with or any error messages associated with it. Targetting a form input without these psuedoclasses MUST cause them to change their value.

## Handling responses for advanced targets
1. A set of updated URLs should be initialized with the URL that was just fetched.
2. If this was a GET request, mark the target element as being loaded from that URL.
3. Process the HTTP response headers as specified in [HTTP response headers](https://tildegit.org/alcinnz/PostJS/src/branch/master/drafts/hyperlinks-advanced-targets.md#http-response-headers) and other specifications.
4. If the `replace` attribute was used, delete the current children of those elements.
5. Parse the response HTML snippet into the target elements. Any `<script>` or `<style>` elements MUST be ignored, but if a `<title>` is encountered the browser chrome must be updated.
6. Update the style for the HTML to resemble what it would've been if these new elements were in there to begin with.
7. Reload any elements loaded from paths conflicting  (to keep the page equally up to date). Upon response these are handled per these steps as if it had a replace target.

For step 7, two paths conflict if one is a directory that contains the other.

However if doing so is better suited to the target output device, user agents MAY perform a full page load instead.

## HTTP response headers
New HTTP response headers are defined for HTTP responses otherwise handled via this specification. These are all prefixed with `X-HTML-` for now.

### X-HTML-Refresh
Specifies additional whitespace-seperated URLs that have been updated and should impact which elements on the page get reloaded.

### X-HTML-SavePoint
Saves the current state and URL of the page to the history stack. and sets the current URL to the specified value. If the user agent doesn't track a history stack there is no need to implement this.

### X-HTML-Remove
Specifies a floating point number of seconds after which the target element should be removed from the page.

## Psuedoclasses
This specification extends CSS selectors to add the following psuedoclasses.

### `:loading`
Once a link with an advanced target attribute is activated, all of it's target elements SHOULD be annotated with the `:loading` psuedoclass. And once each one has been fully populated with it's new content, the `:loading` psuedoclass MUST be removed from it.

### `:removed`
Before any element is removed from the page, the `:removed` psuedoclass SHOULD be added to it. And only once any animations have completed should it be visually removed. If no animations are in effect upon applying this psuedoelement it MUST be removed from the rendered page immediately.

## Security Concerns
Because HTTP requests can be used to invade a user's privacy, user agents MUST communicate what information will be uploaded where before users trigger them. At the same time constantly confirming the user's intention will just lead them to ignore the user agent's warning, so it most communicate unobtrusively enough that it doesn't harm their workflow. They SHOULD also communicate where the response will be communicated.

For devices which can't unobtrusively communicate request information prior to the request the visual design of webpages is constrained to communicate the basic safety information.

### XSS Attacks
There are two main mitigations user agents MUST take against cross-site-scripting attacks. The first is that no links should be allowed to change the contents of an iframe, as far as the CSS selectors are concerned an iframes contents MUST be seperate from the page.

Second to prevent attacks mounted from the HTML snippet being embedded, all `<script>` and `<style>` tags MUST be ignored because they can be used to mutate the embedding page. And user agents MUST make sure that markup being embedded cannot escape the target element it's being loaded into.